@@ -62,6 +62,94 @@ clear_entry_validation_error (GtkEntry *entry)
         g_object_set (entry, "caps-lock-warning", TRUE, NULL);
 }
 
+void
+popup_menu_below_button (GtkMenu   *menu,
+                         gint      *x,
+                         gint      *y,
+                         gboolean  *push_in,
+                         GtkWidget *button)
+{
+        GtkRequisition menu_req;
+        GtkTextDirection direction;
+        GtkAllocation allocation;
+
+        gtk_widget_get_preferred_size (GTK_WIDGET (menu), NULL, &menu_req);
+
+        direction = gtk_widget_get_direction (button);
+
+        gdk_window_get_origin (gtk_widget_get_window (button), x, y);
+        gtk_widget_get_allocation (button, &allocation);
+        *x += allocation.x;
+        *y += allocation.y + allocation.height;
+
+        if (direction == GTK_TEXT_DIR_LTR)
+                *x += MAX (allocation.width - menu_req.width, 0);
+        else if (menu_req.width > allocation.width)
+                *x -= menu_req.width - allocation.width;
+
+        *push_in = TRUE;
+}
+
+void
+down_arrow (GtkStyleContext *context,
+            cairo_t         *cr,
+            gint             x,
+            gint             y,
+            gint             width,
+            gint             height)
+{
+        GtkStateFlags flags;
+        GdkRGBA fg_color;
+        GdkRGBA outline_color;
+        gdouble vertical_overshoot;
+        gint diameter;
+        gdouble radius;
+        gdouble x_double, y_double;
+        gdouble angle;
+        gint line_width;
+
+        flags = gtk_style_context_get_state (context);
+
+        gtk_style_context_get_color (context, flags, &fg_color);
+        gtk_style_context_get_border_color (context, flags, &outline_color);
+
+        line_width = 1;
+        angle = G_PI / 2;
+        vertical_overshoot = line_width / 2.0 * (1. / tan (G_PI / 8));
+        if (line_width % 2 == 1)
+                vertical_overshoot = ceil (0.5 + vertical_overshoot) - 0.5;
+        else
+                vertical_overshoot = ceil (vertical_overshoot);
+        diameter = (gint) MAX (3, width - 2 * vertical_overshoot);
+        diameter -= (1 - (diameter + line_width) % 2);
+        radius = diameter / 2.;
+        x_double = floor ((x + width / 2) - (radius + line_width) / 2.) + (radius + line_width) / 2.;
+
+        y_double = (y + height / 2) - 0.5;
+
+        cairo_save (cr);
+
+        cairo_translate (cr, x_double, y_double);
+        cairo_rotate (cr, angle);
+
+        cairo_move_to (cr, - radius / 2., - radius);
+        cairo_line_to (cr,   radius / 2.,   0);
+        cairo_line_to (cr, - radius / 2.,   radius);
+
+        cairo_close_path (cr);
+
+        cairo_set_line_width (cr, line_width);
+
+        gdk_cairo_set_source_rgba (cr, &fg_color);
+
+        cairo_fill_preserve (cr);
+
+        gdk_cairo_set_source_rgba (cr, &outline_color);
+        cairo_stroke (cr);
+
+        cairo_restore (cr);
+}
+
 #define MAXNAMELEN  (UT_NAMESIZE - 1)
 
 static gboolean
@@ -81,11 +169,30 @@ is_username_used (const gchar *username)
 gboolean
 is_valid_name (const gchar *name)
 {
-        gboolean valid;
+        gboolean is_empty = TRUE;
+        const gchar *c;
 
-        valid = (strlen (name) > 0);
+        /* Valid names must contain:
+         *   1) at least one character.
+         *   2) at least one non-"space" character.
+         */
+        for (c = name; *c; c++) {
+                gunichar unichar;
 
-        return valid;
+                unichar = g_utf8_get_char_validated (c, -1);
+
+                /* Partial UTF-8 sequence or end of string */
+                if (unichar == (gunichar) -1 || unichar == (gunichar) -2)
+                        break;
+
+                /* Check for non-space character */
+                if (!g_unichar_isspace (unichar)) {
+                        is_empty = FALSE;
+                        break;
+                }
+        }
+
+        return !is_empty;
 }
 
 gboolean
@@ -136,7 +243,7 @@ is_valid_username (const gchar *username, gchar **tip)
                         *tip = g_strdup (_("The username cannot start with a '-'"));
                 }
                 else {
-                        *tip = g_strdup (_("The username must consist of:\n"
+                        *tip = g_strdup (_("The username must only consist of:\n"
                                           " \xe2\x9e\xa3 letters from the English alphabet\n"
                                           " \xe2\x9e\xa3 digits\n"
                                           " \xe2\x9e\xa3 any of the characters '.', '-' and '_'"));
@@ -153,7 +260,7 @@ void
 generate_username_choices (const gchar  *name,
                            GtkListStore *store)
 {
-        gboolean in_use;
+        gboolean in_use, same_as_initial;
         char *lc_name, *ascii_name, *stripped_name;
         char **words1;
         char **words2 = NULL;
@@ -301,7 +408,8 @@ generate_username_choices (const gchar  *name,
         }
 
         in_use = is_username_used (item1->str);
-        if (nwords2 > 0 && !in_use && !g_ascii_isdigit (item1->str[0])) {
+        same_as_initial = (g_strcmp0 (item0->str, item1->str) == 0);
+        if (!same_as_initial && nwords2 > 0 && !in_use && !g_ascii_isdigit (item1->str[0])) {
                 gtk_list_store_append (store, &iter);
                 gtk_list_store_set (store, &iter, 0, item1->str, -1);
                 g_hash_table_insert (items, item1->str, item1->str);
