@@ -13,9 +13,7 @@
  * General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * along with this program; if not, see <http://www.gnu.org/licenses/>.
  *
  * Written by:
  *     Jasper St. Pierre <jstpierre@mecheye.net>
@@ -42,6 +40,10 @@
 #define SERVICE_NAME "gdm-password"
 
 struct _GisSummaryPagePrivate {
+  GtkWidget *start_button;
+  GtkWidget *start_button_label;
+  GtkWidget *tagline;
+
   ActUser *user_account;
   const gchar *user_password;
 };
@@ -49,9 +51,6 @@ typedef struct _GisSummaryPagePrivate GisSummaryPagePrivate;
 
 G_DEFINE_TYPE_WITH_PRIVATE (GisSummaryPage, gis_summary_page, GIS_TYPE_PAGE);
 
-#define OBJ(type,name) ((type)gtk_builder_get_object(GIS_PAGE(page)->builder,(name)))
-#define WID(name) OBJ(GtkWidget*,name)
-
 static gboolean
 connect_to_gdm (GdmGreeter      **greeter,
                 GdmUserVerifier **user_verifier)
@@ -273,29 +272,6 @@ gis_summary_page_shown (GisPage *page)
                                    &priv->user_password);
 }
 
-static GtkBuilder *
-gis_summary_page_get_builder (GisPage *page)
-{
-  GtkBuilder *builder = gtk_builder_new ();
-
-  char *filename = g_build_filename (UIDIR, "summary-distro.ui", NULL);
-  GError *error = NULL;
-
-  if (gtk_builder_add_from_file (builder, filename, &error))
-    goto out;
-
-  if (!g_error_matches (error, G_FILE_ERROR, G_FILE_ERROR_NOENT))
-    g_warning ("Error while loading summary override: %s", error->message);
-
-  g_clear_error (&error);
-
-  builder = GIS_PAGE_CLASS (gis_summary_page_parent_class)->get_builder (page);
-
- out:
-  g_free (filename);
-  return builder;
-}
-
 static char *
 get_item (const char *buffer, const char *name)
 {
@@ -332,8 +308,10 @@ get_item (const char *buffer, const char *name)
 static void
 update_distro_name (GisSummaryPage *page)
 {
+  GisSummaryPagePrivate *priv = gis_summary_page_get_instance_private (page);
   char *buffer;
   char *name;
+  char *text;
 
   name = NULL;
 
@@ -343,26 +321,36 @@ update_distro_name (GisSummaryPage *page)
       g_free (buffer);
     }
 
-  if (name)
-    {
-      gchar *label;
-      label = g_strdup_printf (_("_Start using %s"), name);
-      gtk_label_set_label (GTK_LABEL (WID ("summary-start-button-label")), label);
-      g_free (label);
-      g_free (name);
-    }
+  if (!name)
+    name = g_strdup ("GNOME 3");
+
+  /* Translators: the parameter here is the name of a distribution,
+   * like "Fedora" or "Ubuntu". It falls back to "GNOME 3" if we can't
+   * detect any distribution. */
+  text = g_strdup_printf (_("_Start using %s"), name);
+  gtk_label_set_label (GTK_LABEL (priv->start_button_label), text);
+  g_free (text);
+
+  /* Translators: the parameter here is the name of a distribution,
+   * like "Fedora" or "Ubuntu". It falls back to "GNOME 3" if we can't
+   * detect any distribution. */
+  text = g_strdup_printf (_("Thank you for choosing %s.\nWe hope that you love it."), name);
+  gtk_label_set_label (GTK_LABEL (priv->tagline), text);
+  g_free (text);
+
+  g_free (name);
 }
 
 static void
 gis_summary_page_constructed (GObject *object)
 {
   GisSummaryPage *page = GIS_SUMMARY_PAGE (object);
+  GisSummaryPagePrivate *priv = gis_summary_page_get_instance_private (page);
 
   G_OBJECT_CLASS (gis_summary_page_parent_class)->constructed (object);
 
-  gtk_container_add (GTK_CONTAINER (page), WID ("summary-page"));
   update_distro_name (page);
-  g_signal_connect (WID("summary-start-button"), "clicked", G_CALLBACK (done_cb), page);
+  g_signal_connect (priv->start_button, "clicked", G_CALLBACK (done_cb), page);
 
   gis_page_set_complete (GIS_PAGE (page), TRUE);
 
@@ -372,27 +360,60 @@ gis_summary_page_constructed (GObject *object)
 static void
 gis_summary_page_locale_changed (GisPage *page)
 {
-  gis_page_set_title (page, _("Thank You"));
+  gis_page_set_title (page, _("Ready to Go"));
   update_distro_name (GIS_SUMMARY_PAGE (page));
 }
 
+static void
+add_style_from_resource (const char *resource)
+{
+  GtkCssProvider *provider;
+  GFile *file;
+  char *uri;
+
+  provider = gtk_css_provider_new ();
+
+  uri = g_strconcat ("resource://", resource, NULL);
+  file = g_file_new_for_uri (uri);
+
+  if (!gtk_css_provider_load_from_file (provider, file, NULL))
+    goto out;
+
+  gtk_style_context_add_provider_for_screen (gdk_screen_get_default (),
+                                             GTK_STYLE_PROVIDER (provider),
+                                             GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
+
+ out:
+  g_object_unref (file);
+  g_free (uri);
+}
+
 static void
 gis_summary_page_class_init (GisSummaryPageClass *klass)
 {
   GisPageClass *page_class = GIS_PAGE_CLASS (klass);
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
+  gtk_widget_class_set_template_from_resource (GTK_WIDGET_CLASS (klass), "/org/gnome/initial-setup/gis-summary-page.ui");
+
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisSummaryPage, start_button);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisSummaryPage, start_button_label);
+  gtk_widget_class_bind_template_child_private (GTK_WIDGET_CLASS (klass), GisSummaryPage, tagline);
+
   page_class->page_id = PAGE_ID;
-  page_class->get_builder = gis_summary_page_get_builder;
   page_class->locale_changed = gis_summary_page_locale_changed;
   page_class->shown = gis_summary_page_shown;
   object_class->constructed = gis_summary_page_constructed;
+
+  add_style_from_resource ("/org/gnome/initial-setup/gis-summary-page.css");
 }
 
 static void
 gis_summary_page_init (GisSummaryPage *page)
 {
   g_resources_register (summary_get_resource ());
+
+  gtk_widget_init_template (GTK_WIDGET (page));
 }
 
 void
